<?php

namespace App\Http\Controllers\Api\Admin;

use App\Http\Controllers\Controller;
use App\Library\FoxPaginate\TablePaginator;
use App\Models\Bot;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\MessageBag;
use JetBrains\PhpStorm\NoReturn;

class CustomerController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        return response()->json([
            'ok' => true,
            'data' => TablePaginator::paginate(User::role('customer')->with('bots'), [
                'per_page' => 10,
                'search_fields' => [
                    'username',
                    'fullname',
                ]
            ])
        ]);
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(Request $request): JsonResponse
    {
        $needles = ['bot_ids', 'fullname', 'username', 'password', 'max_channels'];
        $errors = Validator::make($request->only($needles), [
            'bot_ids' => 'array',
            'bot_ids.*' => 'required|exists:bots,_id',
            'fullname' => 'required',
            'username' => 'required|string|regex:/\w+$/|max:255|unique:users,username',
            'password' => 'required|min:8',
            'max_channels' => 'required|integer|min:1',
        ], [
            'bot_ids.*' => 'ربات انتخابی یافت نشد.',
            'fullname.*' => 'نام به درستی وارد نشده.',
            'username.*' => 'نام کاربری به درستی وارد نشده یا موجود است.',
            'password.*' => 'کلمه عبور صحیح نیست حداقل 8 حرف.',
            'max_channels.*' => 'حداکثر کانال مجاز صحیح نیست.',
        ])->errors();

        if ($errors->isEmpty()) {
            $data = $request->only($needles);
            $data['password'] = Hash::make($data['password']);
            $data['max_channels'] = (int)$data['max_channels'];
            $bots = collect($data['bot_ids'])
                ->flatten()
                ->map(function ($id) {
                    return Bot::query()->find($id);
                })->all();
            unset($data['bot_ids']);

            $user = new User($data);
            if ($user->save()) {
                $user->bots()->saveMany($bots);
                $user->assignRole('customer');
            } else
                $errors = new MessageBag([
                    'unknown' => ['خطا غیرمنتظره!']
                ]);
        }

        if ($errors->isEmpty()) {
            return response()->json([
                'ok' => true
            ]);
        } else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param User $customer
     * @return JsonResponse
     */
    public function update(Request $request, User $customer): JsonResponse
    {
        $needles = ['bot_ids', 'max_channels'];
        $errors = Validator::make($request->only($needles), [
            'bot_ids' => 'array',
            'bot_ids.*' => 'required|exists:bots,_id',
            'max_channels' => 'required|integer|min:1',
        ], [
            'bot_ids.*' => 'ربات انتخابی یافت نشد.',
            'max_channels.*' => 'حداکثر کانال مجاز صحیح نیست.',
        ])->errors();

        if ($errors->isEmpty()) {
            $data = $request->only($needles);
            $data['max_channels'] = (int)$data['max_channels'];
            $bots = collect($data['bot_ids'])
                ->flatten()
                ->map(function ($id) {
                    return Bot::query()->find($id);
                })->all();
            unset($data['bot_ids']);


            if ($customer->update($data)) {
                $customer->bots()->sync([]);
                $customer->bots()->saveMany($bots);
            } else
                $errors = new MessageBag([
                    'unknown' => ['خطا غیرمنتظره!']
                ]);
        }

        if ($errors->isEmpty()) {
            return response()->json([
                'ok' => true
            ]);
        } else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param User $customer
     * @return JsonResponse
     */
    public function destroy(User $customer): JsonResponse
    {
        $errors = new MessageBag();
        if ($customer->hasRole('customer')) {
            if (!$customer->delete())
                $errors = new MessageBag([
                    'unknown' => ['خطا در عملیات!']
                ]);
        }
        if ($errors->isEmpty())
            return response()->json([
                'ok' => true
            ]);
        else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 500);
    }
}
