<?php

namespace App\Http\Controllers\Api\Customer;

use App\Http\Controllers\Controller;
use App\Library\FoxTg\FoxTgApi;
use App\Models\Bot;
use App\Models\Channel;
use App\Models\User;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class ChannelController extends Controller
{
    public function index(Request $request): JsonResponse
    {
        /** @var User $user */
        $user = auth()->user();
        return response()->json([
            'ok' => true,
            'data' => $user->channels()->with('bots')->whereIn('bot_ids', [$request->get('bot_id')])->get()
        ]);
    }

    public function store(Request $request): JsonResponse
    {
        $needles = ['bot_id', 'link'];
        $errors = Validator::make($request->only($needles), [
            'bot_id' => 'required|exists:bots,_id',
            'link' => 'required|string|min:4',
        ], [
            'bot_id.*' => 'ربات انتخابی یافت نشد.',
            'link.*' => 'لینک کانال صحیح نیست.',
        ])->errors();

        if ($errors->isEmpty()) {
            /** @var $bot Bot */
            $bot = Bot::query()->find($request->post('bot_id'));
            $tg_helper = new FoxTgApi(env('BOT_API'));
            try {
                $result = $tg_helper->call([
                    'method' => 'post',
                    'action' => 'channels/join',
                    'data' => [
                        'session' => $bot['session_key'],
                        'channel' => $request->post('link'),
                    ]
                ]);
                if ($result->status() === 200) {
                    $response = $result->response();
                    /** @var $customer User */
                    $customer = auth()->user();

                    $channel = Channel::query()->updateOrCreate([
                        '_id' => $response['data']['chat']['id'],
                    ], [
                        'access_hash' => $response['data']['chat']['accessHash'],
                        'title' => $response['data']['chat']['title'],
                    ]);
                    $bot->channels()->attach($channel['_id']);
                    $customer->channels()->attach($channel['_id']);

                    return response()->json([
                        'ok' => true
                    ]);
                } else
                    return response()->json($result->response(), $result->status());
            } catch (GuzzleException $e) {
                return response()->json([
                    'ok' => false,
                    'errors' => [
                        'connection' => [
                            'CONNECTION_FAILED'
                        ]]
                ], $e->getCode());
            }
        } else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
    }

    public function syncChannels(Request $request): JsonResponse
    {
        $needles = ['bot_id', 'type'];
        $errors = Validator::make($request->only($needles), [
            'bot_id' => 'required|exists:bots,_id',
            'private' => 'boolean',
        ], [
            'bot_id.*' => 'ربات انتخابی یافت نشد.',

        ])->errors();

        if ($errors->isEmpty()) {
            /** @var $bot Bot */
            $bot = Bot::query()->find($request->post('bot_id'));
            $tg_helper = new FoxTgApi(env('BOT_API'));
            try {
                $result = $tg_helper->call([
                    'method' => 'get',
                    'action' => 'channels/',
                    'data' => [
                        'session_key' => $bot['session_key'],
                        'type' => $request->post('private') ? 'private' : 'public',
                    ]
                ]);
                if ($result->status() === 200) {
                    $response = $result->response();
                    /** @var $customer User */
                    $customer = auth()->user();

                    /** add data */
                    $add_data = $response['data']['add'];
                    foreach ($add_data as $item) {
                        $channel = Channel::query()->updateOrCreate([
                            '_id' => $item['id'],
                        ], [
                            'access_hash' => $item['accessHash'],
                            'title' => $item['title'],
                        ]);
                        $bot->channels()->attach($channel['_id']);
                        $customer->channels()->attach($channel['_id']);
                    }
                    /** remove data */
                    $remove_data = $response['data']['remove'];
                    if (count($remove_data) > 0) {
                        $bot->channels()->detach($remove_data);
                        $customer->channels()->detach($remove_data);
                    }

                    return response()->json([
                        'ok' => true
                    ]);
                } else
                    return response()->json($result->response(), $result->status());
            } catch (GuzzleException $e) {
                return response()->json([
                    'ok' => false,
                    'errors' => [
                        'connection' => [
                            'CONNECTION_FAILED'
                        ]]
                ], $e->getCode());
            }
        } else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
    }
}
